package lcsb.mapviewer.common.geometry;

import java.awt.geom.Point2D;

/**
 * Implementation of the Point2D that does not allow to modify data. Useful for
 * getters of data that should not be modified by chance.
 * 
 * @author Piotr Gawron
 *
 */
final public class ImmutablePoint2D extends Point2D {
  final private double x;
  final private double y;

  public ImmutablePoint2D(double x, double y) {
    this.x = x;
    this.y = y;
  }

  public ImmutablePoint2D(Point2D point) {
    this(point.getX(), point.getY());
  }

  @Override
  public double getX() {
    return x;
  }

  @Override
  public double getY() {
    return y;
  }

  @Override
  public void setLocation(double x, double y) {
    throw new UnsupportedOperationException();
  }

}
