package lcsb.mapviewer.converter.graphics.bioEntity.element.species;

import java.awt.*;
import java.awt.geom.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import lcsb.mapviewer.commands.ColorExtractor;
import lcsb.mapviewer.converter.graphics.ConverterParams;
import lcsb.mapviewer.model.map.species.Phenotype;

/**
 * This class defines methods used for drawing SpeciesAlias of {@link Phenotype}
 * on the {@link Graphics2D} object.
 * 
 * @author Piotr Gawron
 * 
 */
public class PhenotypeConverter extends SpeciesConverter<Phenotype> {
  /**
   * Default class logger.
   */
  @SuppressWarnings("unused")
  private static Logger logger = LogManager.getLogger();

  /**
   * Default constructor.
   * 
   * @param colorExtractor
   *          Object that helps to convert {@link ColorSchema} values into colors
   *          when drawing elements
   * 
   */
  public PhenotypeConverter(ColorExtractor colorExtractor) {
    super(colorExtractor);
  }

  @Override
  protected void drawImpl(Phenotype phenotype, final Graphics2D graphics, final ConverterParams params) {
    GeneralPath path = getShape(phenotype);

    Color oldColor = graphics.getColor();
    graphics.setColor(phenotype.getFillColor());
    graphics.fill(path);
    graphics.setColor(phenotype.getBorderColor());
    Stroke stroke = graphics.getStroke();
    graphics.setStroke(getBorderLine(phenotype));
    graphics.draw(path);
    graphics.setStroke(stroke);
    drawText(phenotype, graphics, params);
    graphics.setColor(oldColor);
  }

  /**
   * Returns shape of the {@link Phenotype} as a {@link GeneralPath} object.
   * 
   * @param phenotype
   *          {@link Phenotype} for which we are looking for a border
   * @return {@link GeneralPath} object defining border of the given
   *         {@link Phenotype}
   */
  @Override
  protected GeneralPath getShape(final Phenotype phenotype) {
    // CHECKSTYLE:OFF
    GeneralPath path = new GeneralPath(Path2D.WIND_EVEN_ODD, 6);
    
    double x = phenotype.getX();
    double y = phenotype.getY();
    double width = phenotype.getWidth();
    double height = phenotype.getHeight();
    
    double cutSize = Math.min(width, height)/2;
    
    path.moveTo(x, y + height / 2);
    path.lineTo(x + cutSize, y);
    path.lineTo(x + width - cutSize, y);
    path.lineTo(x + width, y + height / 2);
    path.lineTo(x + width - cutSize, y + height);
    path.lineTo(x + cutSize, y + height);
    
    // CHECKSTYLE:ON
    path.closePath();
    return path;
  }

  @Override
  public PathIterator getBoundPathIterator(Phenotype phenotype) {
    return getShape(phenotype).getPathIterator(new AffineTransform());
  }

}
