package lcsb.mapviewer.model.overlay;

import java.io.Serializable;
import java.util.*;

import javax.persistence.*;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;

import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.cache.UploadedFileEntry;
import lcsb.mapviewer.model.map.layout.ReferenceGenomeType;
import lcsb.mapviewer.model.user.User;

@Entity
public class DataOverlay implements Serializable {

  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  public static final Comparator<? super DataOverlay> ORDER_COMPARATOR = new Comparator<DataOverlay>() {

    @Override
    public int compare(DataOverlay o1, DataOverlay o2) {
      if (Objects.equals(o1.isPublic(), o2.isPublic())) {
        return o1.getOrderIndex() - o2.getOrderIndex();
      } else {
        if (o1.isPublic()) {
          return 1;
        } else {
          return -1;
        }
      }
    }
  };

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private int id;

  @Column(length = 255, nullable = false)
  private String name;

  private boolean isPublic = false;

  private boolean googleLicenseConsent = false;

  /**
   * Data overlay type. It can be null in such case it should be obtained from
   * {@link #inputData}.
   */
  @Column(nullable = false)
  @Enumerated(EnumType.STRING)
  private DataOverlayType colorSchemaType = null;

  /**
   * Project to which this data overlay is assigned.
   */
  @ManyToOne(fetch = FetchType.LAZY, optional = false)
  private Project project;

  /**
   * Who created the layout.
   */
  @ManyToOne(fetch = FetchType.LAZY, optional = false)
  private User creator;

  private int orderIndex = 0;

  /**
   * Short description used for this layout.
   */
  @Column(columnDefinition = "TEXT")
  private String description;

  @Enumerated(EnumType.STRING)
  private ReferenceGenomeType genomeType;

  private String genomeVersion;

  /**
   * Here we store input file.
   */
  @Cascade({ CascadeType.SAVE_UPDATE })
  @OneToOne(fetch = FetchType.LAZY, optional = false)
  @JoinColumn(name = "file_entry_id")
  private UploadedFileEntry inputData;

  @Cascade({ CascadeType.SAVE_UPDATE })
  @OneToMany(mappedBy = "dataOverlay", orphanRemoval = true)
  private Set<DataOverlayEntry> entries = new HashSet<>();

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public boolean isGoogleLicenseConsent() {
    return googleLicenseConsent;
  }

  public void setGoogleLicenseConsent(boolean googleLicenseConsent) {
    this.googleLicenseConsent = googleLicenseConsent;
  }

  public DataOverlayType getColorSchemaType() {
    return colorSchemaType;
  }

  public void setColorSchemaType(DataOverlayType colorSchemaType) {
    this.colorSchemaType = colorSchemaType;
  }

  public Project getProject() {
    return project;
  }

  public void setProject(Project project) {
    this.project = project;
  }

  public User getCreator() {
    return creator;
  }

  public void setCreator(User creator) {
    this.creator = creator;
  }

  public int getOrderIndex() {
    return orderIndex;
  }

  public void setOrderIndex(int orderIndex) {
    this.orderIndex = orderIndex;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public UploadedFileEntry getInputData() {
    return inputData;
  }

  public void setInputData(UploadedFileEntry inputData) {
    this.inputData = inputData;
  }

  public int getId() {
    return id;
  }

  public boolean isPublic() {
    return isPublic;
  }

  public void setPublic(boolean isPublic) {
    this.isPublic = isPublic;
  }

  public Set<DataOverlayEntry> getEntries() {
    return entries;
  }

  public void setEntries(Set<DataOverlayEntry> entries) {
    this.entries = entries;
  }

  public void addEntry(DataOverlayEntry dataOverlayEntry) {
    this.entries.add(dataOverlayEntry);
    dataOverlayEntry.setDataOverlay(this);
  }

  public void addEntries(Collection<DataOverlayEntry> entries) {
    for (DataOverlayEntry dataOverlayEntry : entries) {
      addEntry(dataOverlayEntry);
    }

  }

  public ReferenceGenomeType getGenomeType() {
    return genomeType;
  }

  public void setGenomeType(ReferenceGenomeType genomeType) {
    this.genomeType = genomeType;
  }

  public String getGenomeVersion() {
    return genomeVersion;
  }

  public void setGenomeVersion(String genomeVersion) {
    this.genomeVersion = genomeVersion;
  }

}
