package lcsb.mapviewer.model.overlay;

import java.util.*;

import javax.persistence.*;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;

import lcsb.mapviewer.common.exception.NotImplementedException;

/**
 * Information about coloring element according to genetic variation
 * information.
 * 
 * @author Piotr Gawron
 *
 */
@Entity
@DiscriminatorValue("GENE_VARIANT_DATA_OVERLAY")
public class GeneVariantDataOverlayEntry extends DataOverlayEntry {

  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  @Cascade({ CascadeType.ALL })
  @OneToMany(mappedBy = "dataOverlayEntry", orphanRemoval = true)
  private Set<GeneVariant> geneVariants = new HashSet<>();

  public GeneVariantDataOverlayEntry() {
  }

  /**
   * Constructor that creates copy of the object.
   * 
   * @param schema
   *          original object from which data will be copied
   */
  public GeneVariantDataOverlayEntry(GeneVariantDataOverlayEntry schema) {
    super(schema);
    this.addGeneVariants(schema.getGeneVariants());
  }

  /**
   * Adds {@link GeneVariant gene variations} to {@link #geneVariants}.
   * 
   * @param geneVariants
   *          collection of {@link GeneVariant} objects to add
   */
  public void addGeneVariants(Collection<GeneVariant> geneVariants) {
    for (GeneVariant geneVariation : geneVariants) {
      addGeneVariant(geneVariation.copy());
    }
  }

  @Override
  public GeneVariantDataOverlayEntry copy() {
    if (this.getClass().equals(GeneVariantDataOverlayEntry.class)) {
      return new GeneVariantDataOverlayEntry(this);
    } else {
      throw new NotImplementedException("Copy not implemented for class: " + this.getClass());
    }
  }

  /**
   * @return the geneVariations
   * @see #geneVariants
   */
  public Set<GeneVariant> getGeneVariants() {
    return geneVariants;
  }

  /**
   * Adds {@link GeneVariant} to list gene variations.
   * 
   * @param gv
   *          object to add
   */
  public void addGeneVariant(GeneVariant gv) {
    this.geneVariants.add(gv);
    gv.setDataOverlayEntry(this);
  }
}
