package lcsb.mapviewer.persist;

import java.sql.Connection;
import java.util.*;

import javax.sql.DataSource;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.flywaydb.core.Flyway;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.init.DatabasePopulator;
import org.springframework.jdbc.datasource.init.ScriptException;
import org.springframework.stereotype.Service;

@Service
public class CustomDatabasePopulator implements DatabasePopulator {

  private Logger logger = LogManager.getLogger();

  private ConfigurationHolder config;

  private DataSource dataSource;

  @Autowired
  public CustomDatabasePopulator(ConfigurationHolder config, DataSource dataSource) {
    this.config = config;
    this.dataSource = dataSource;
  }

  @Override
  public void populate(Connection connection) throws ScriptException {
    String migrationFolder = "postgres";
    if (config.getDbDriver().equals("org.hsqldb.jdbcDriver")) {
      migrationFolder = "hsql";
    }
    List<String> locations = new ArrayList<>(
        Arrays.asList("classpath:lcsb.mapviewer.persist/migration/" + migrationFolder,
            "classpath:db.migration." + migrationFolder));
    if (config.getPathToDataScripts() != null) {
      logger.info("Additional SQL scripts located at: " + config.getPathToDataScripts());
      locations.add("filesystem:" + config.getPathToDataScripts());
    }
    Flyway.configure().dataSource(config.getDbUri(), config.getDbUsername(), config.getDbPassword())
        .baselineVersion("12.1.0")
        .locations(locations.toArray(new String[0]))
        .outOfOrder(true)
        .baselineOnMigrate(true)
        .validateOnMigrate(false)
        .load()
        .migrate();
  }

  public DataSource getDataSource() {
    return dataSource;
  }

  public void setDataSource(DataSource dataSource) {
    this.dataSource = dataSource;
  }

}
