package lcsb.mapviewer.persist;

import java.awt.Color;

import org.junit.Rule;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import lcsb.mapviewer.common.UnitTestFailedWatcher;
import lcsb.mapviewer.common.geometry.PointTransformation;
import lcsb.mapviewer.model.graphics.*;
import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.layout.graphics.*;
import lcsb.mapviewer.model.map.reaction.*;
import lcsb.mapviewer.model.map.reaction.type.ModulationReaction;
import lcsb.mapviewer.model.map.reaction.type.TransportReaction;
import lcsb.mapviewer.model.map.species.*;

@ContextConfiguration(classes = SpringPersistTestConfig.class)
@RunWith(SpringJUnit4ClassRunner.class)
public class PersistTestFunctionsNoTransaction {

  protected static String ADMIN_BUILT_IN_LOGIN = "admin";
  
  protected static String TEST_PROJECT_ID = "Some_id";
  
  @Rule
  public UnitTestFailedWatcher unitTestFailedWatcher = new UnitTestFailedWatcher();
  
  private static int identifierCounter = 0;
  
  private PointTransformation pt = new PointTransformation();
  
  private static int zIndex = 0;

  protected static GenericProtein createProtein() {
    return createProtein(0, 0, 100, 200, "s" + (identifierCounter++));
  }

  protected Compartment createCompartment(double x, double y, double width, double height, String compartmentId) {
    Compartment compartment = new Compartment(compartmentId);
    assignCoordinates(x, y, width, height, compartment);
    return compartment;
  }

  protected Compartment createCompartment(String compartmentId) {
    return createCompartment(10, 20, 100, 200, compartmentId);
  }

  protected Compartment createCompartment() {
    return createCompartment("c" + (identifierCounter++));
  }

  protected static GenericProtein createProtein(String proteinId) {
    return createProtein(0, 0, 100, 200, proteinId);
  }

  protected static GenericProtein createProtein(double x, double y, double width, double height, String proteinId) {
    GenericProtein protein = new GenericProtein("s" + identifierCounter++);
    protein.setElementId(proteinId);
    assignCoordinates(x, y, width, height, protein);
    return protein;
  }

  private static void assignCoordinates(double x, double y, double width, double height, Element protein) {
    protein.setX(x);
    protein.setY(y);
    protein.setZ(zIndex++);
    protein.setWidth(width);
    protein.setHeight(height);
    protein.setNameX(x);
    protein.setNameY(y);
    protein.setNameWidth(width);
    protein.setNameHeight(height);
    protein.setNameVerticalAlign(VerticalAlign.MIDDLE);
    protein.setNameHorizontalAlign(HorizontalAlign.CENTER);
  }

  protected static Ion createIon(String ionId) {
    Ion ion = new Ion(ionId);
    assignCoordinates(1, 2, 10, 20, ion);
    return ion;
  }

  protected Complex createComplex(double x, double y, double width, double height, String complexId) {
    Complex complex = new Complex(complexId);
    assignCoordinates(x, y, width, height, complex);
    return complex;
  }

  protected AntisenseRna createAntisenseRna(String aRnaId) {
    AntisenseRna antisenseRna = new AntisenseRna(aRnaId);
    assignCoordinates(1, 2, 10, 20, antisenseRna);
    return antisenseRna;
  }

  public PersistTestFunctionsNoTransaction() {
    super();
  }

  protected Phenotype createPhenotype(String phenotypeId) {
    Phenotype phenotype = new Phenotype(phenotypeId);
    assignCoordinates(1, 2, 10, 20, phenotype);
    return phenotype;
  }

  protected Rna createRna(String rnaId) {
    Rna rna = new Rna(rnaId);
    assignCoordinates(1, 2, 10, 20, rna);
  
    return rna;
  }

  protected TransportReaction createReaction(Element reactantElement, Element productElement) {
    TransportReaction result = new TransportReaction("re" + identifierCounter++);
    addLines(reactantElement, productElement, result);
    return result;
  }

  private void addLines(Element reactantElement, Element productElement, Reaction result) {
    Product product = new Product(productElement);
    product.setLine(new PolylineData(productElement.getCenter(),
        pt.getPointOnLine(reactantElement.getCenter(), productElement.getCenter(), 0.6)));
    result.addProduct(product);
    Reactant reactant = new Reactant(reactantElement);
    reactant.setLine(new PolylineData(productElement.getCenter(),
        pt.getPointOnLine(reactantElement.getCenter(), productElement.getCenter(), 0.4)));
    result.addReactant(reactant);
    result.setZ(zIndex++);
    result.setLine(new PolylineData(pt.getPointOnLine(reactantElement.getCenter(), productElement.getCenter(), 0.4),
        pt.getPointOnLine(reactantElement.getCenter(), productElement.getCenter(), 0.6)));
  }

  protected ModulationReaction createModulationReaction(Element reactantElement, Element productElement) {
    ModulationReaction result = new ModulationReaction("re" + identifierCounter++);
    addLines(reactantElement, productElement, result);
    result.setZ(zIndex++);
    return result;
  }

  protected LayerRect createRect() {
    LayerRect lr = new LayerRect();
    lr.setBorderColor(Color.YELLOW);
    lr.setZ(0);
    return lr;
  }

  protected LayerOval createOval() {
    LayerOval oval = new LayerOval();
    oval.setColor(Color.GREEN);
    oval.setZ(0);
    return oval;
  }

  protected LayerText createText() {
    LayerText text = new LayerText();
    text.setColor(Color.GREEN);
    text.setZ(0);
    return text;
  }

}