package lcsb.mapviewer.persist.dao.map;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelFullIndexed;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Protein;
import lcsb.mapviewer.persist.DbUtils;
import lcsb.mapviewer.persist.PersistTestFunctionsNoTransaction;

public class ReactionDaoTestInSeparateThread extends PersistTestFunctionsNoTransaction {

  Logger logger = LogManager.getLogger();

  @Autowired
  private ReactionDao reactionDao;

  @Autowired
  private ModelDao modelDao;

  @Autowired
  private DbUtils dbUtils;

  @Test
  public void testGetByCoordinates() {
    Model map = new ModelFullIndexed(null);
    Protein p1 = createProtein();
    Protein p2 = createProtein();
    map.addElement(p1);
    map.addElement(p2);

    Reaction r1 = createReaction(p1, p2);
    r1.getReactants().get(0).getLine().setLine(0, 100, 100, 200, 200);
    r1.getProducts().get(0).getLine().setLine(0, 200, 200, 200, 300);
    r1.getLine().setLine(0, 150, 150, 250, 250);
    Reaction r2 = createReaction(p1, p2);
    r2.getReactants().get(0).getLine().setLine(0, 50, 50, 50, 0);
    r2.getProducts().get(0).getLine().setLine(0, 50, 0, 30, 0);
    r2.getLine().setLine(0, 50, 25, 40, 0);
    Reaction r3 = createReaction(p1, p2);
    r3.getReactants().get(0).getLine().setLine(0, 500, 500, 500, 600);
    r3.getProducts().get(0).getLine().setLine(0, 500, 600, 500, 700);
    r3.getLine().setLine(0, 500, 550, 500, 650);
    map.addReaction(r1);
    map.addReaction(r2);
    map.addReaction(r3);
    r1.addSynonym("syn1");
    r1.addSynonym("syn2");
    r1.addSynonym("syn3");

    dbUtils.createSessionForCurrentThread();
    modelDao.add(map);
    dbUtils.closeSessionForCurrentThread();

    dbUtils.createSessionForCurrentThread();
    List<Reaction> reactions = reactionDao.getByCoordinates(map.getModelData(), new Point2D.Double(0, 0), 100,
        new ArrayList<>());
    
    assertEquals(r2.getElementId(), reactions.get(0).getElementId());
    assertEquals(r1.getElementId(), reactions.get(1).getElementId());
    assertEquals(r3.getElementId(), reactions.get(2).getElementId());
    
    assertNotEquals(r1, reactions.get(1));
    dbUtils.closeSessionForCurrentThread();
  }
}
