package lcsb.mapviewer.api.projects.models.bioEntities;

import java.awt.geom.Point2D;
import java.util.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.model.map.BioEntity;
import lcsb.mapviewer.services.QueryException;
import lcsb.mapviewer.services.interfaces.ISearchService;

@Transactional
@Service
public class BioEntitiesRestImpl extends BaseRestImpl {

  /**
   * Default class logger.
   */
  @SuppressWarnings("unused")
  private Logger logger = LogManager.getLogger();

  private ISearchService searchService;

  @Autowired
  public BioEntitiesRestImpl(ISearchService searchService) {
    this.searchService = searchService;
  }

  public List<Map<String, Object>> getClosestElementsByCoordinates(
      String projectId, String modelId, Point2D coordinates, Integer count, String perfectMatch, String type)
      throws QueryException {
    List<Map<String, Object>> resultMap = new ArrayList<>();

    Set<String> types = new LinkedHashSet<>();
    if (!type.isEmpty()) {
      for (String str : type.split(",")) {
        types.add(str.toLowerCase());
      }
    }
    int mapId = parseInteger(modelId, "modelId");

    List<BioEntity> elements = searchService.getClosestElements(projectId, mapId, coordinates, count,
        perfectMatch.equalsIgnoreCase("true"), types);
    for (BioEntity object : elements) {
      Map<String, Object> result = createMinifiedSearchResult(object);
      resultMap.add(result);
    }
    return resultMap;
  }

  public List<Map<String, Object>> getElementsByQuery(String projectId, String query, String modelId,
      Integer maxElements, String perfectMatch) throws QueryException {
    List<Map<String, Object>> resultMap = new ArrayList<>();

    boolean match = perfectMatch.equals("true");
    List<BioEntity> elements = searchService.searchByQuery(projectId, modelId, query,
        maxElements, match);
    for (BioEntity object : elements) {
      Map<String, Object> result = createMinifiedSearchResult(object);
      resultMap.add(result);
    }
    return resultMap;
  }

  public List<String> getSuggestedQueryList(String projectId) throws QueryException {
    return searchService.getSuggestedQueryList(projectId);
  }

}
