package lcsb.mapviewer.api.projects.models.parameters;

import java.util.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.model.map.kinetics.SbmlParameter;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.QueryException;
import lcsb.mapviewer.services.interfaces.IReactionService;

@Transactional
@Service
public class ParametersRestImpl extends BaseRestImpl {

  @SuppressWarnings("unused")
  private Logger logger = LogManager.getLogger();

  @Autowired
  private IReactionService reactionService;

  public Map<String, Object> getParameter(String projectId, String modelId, String parameterId)
      throws QueryException {
    int id = parseInteger(parameterId, "parameterId");
    for (SbmlParameter parameter : getGlobalParametersFromProject(projectId, modelId)) {
      if (parameter.getId() == id) {
        return parameterToMap(parameter, true);
      }
    }
    for (SbmlParameter parameter : getReactionParametersFromProject(projectId, modelId)) {
      if (parameter.getId() == id) {
        return parameterToMap(parameter, false);
      }
    }
    throw new ObjectNotFoundException("Parameter with given id doesn't exist");
  }

  private List<SbmlParameter> getReactionParametersFromProject(String projectId, String modelId)
      throws QueryException {
    return reactionService.getParameters(projectId, modelId);
  }

  private Collection<SbmlParameter> getGlobalParametersFromProject(String projectId, String modelId)
      throws QueryException {
    return getModelService().getParametersForModel(projectId, modelId);
  }

  private Map<String, Object> parameterToMap(SbmlParameter parameter, boolean global) {
    Map<String, Object> result = new TreeMap<>();
    result.put("id", parameter.getId());
    result.put("parameterId", parameter.getParameterId());
    result.put("name", parameter.getName());
    result.put("value", parameter.getValue());
    result.put("global", global);
    if (parameter.getUnits() != null) {
      result.put("unitsId", parameter.getUnits().getId());
    } else {
      result.put("unitsId", null);
    }
    return result;
  }

  public List<Map<String, Object>> getParameters(String projectId, String modelId)
      throws QueryException {
    List<Map<String, Object>> result = new ArrayList<>();
    for (SbmlParameter parameter : getGlobalParametersFromProject(projectId, modelId)) {
      result.add(parameterToMap(parameter, true));
    }
    for (SbmlParameter parameter : getReactionParametersFromProject(projectId, modelId)) {
      result.add(parameterToMap(parameter, false));
    }
    return result;
  }
}
