package lcsb.mapviewer.api.projects.models.units;

import java.util.*;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.model.map.kinetics.SbmlUnit;
import lcsb.mapviewer.model.map.kinetics.SbmlUnitTypeFactor;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.QueryException;

@Transactional
@Service
public class UnitsRestImpl extends BaseRestImpl {

  /**
   * Default class logger.
   */
  @SuppressWarnings("unused")
  private Logger logger = LogManager.getLogger();

  public Map<String, Object> getUnit(String projectId, String modelId, String unitId)
      throws QueryException {
    int id = parseInteger(unitId, "unitId");
    for (SbmlUnit unit : getModelService().getUnitsForModel(projectId, modelId)) {
      if (unit.getId() == id) {
        return unitToMap(unit);
      }
    }
    throw new ObjectNotFoundException("Unit with given id doesn't exist");
  }

  private Map<String, Object> unitToMap(SbmlUnit unit) {
    Map<String, Object> result = new TreeMap<>();
    result.put("id", unit.getId());
    result.put("unitId", unit.getUnitId());
    result.put("name", unit.getName());
    List<Map<String, Object>> factors = new ArrayList<>();
    for (SbmlUnitTypeFactor factor : unit.getUnitTypeFactors()) {
      factors.add(factorToMap(factor));
    }
    result.put("unitTypeFactors", factors);
    return result;
  }

  private Map<String, Object> factorToMap(SbmlUnitTypeFactor factor) {
    Map<String, Object> result = new TreeMap<>();
    result.put("id", factor.getId());
    result.put("exponent", factor.getExponent());
    result.put("multiplier", factor.getMultiplier());
    result.put("scale", factor.getScale());
    result.put("unitType", factor.getUnitType());
    return result;
  }

  public List<Map<String, Object>> getUnits(String projectId, String modelId) throws QueryException {
    List<Map<String, Object>> result = new ArrayList<>();
    for (SbmlUnit unit : getModelService().getUnitsForModel(projectId, modelId)) {
      result.add(unitToMap(unit));
    }
    return result;
  }
}
