package lcsb.mapviewer.api.projects.models;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.*;

import org.junit.*;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.converter.graphics.PdfImageGenerator;
import lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.cache.FileEntry;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.persist.dao.ProjectDao;
import lcsb.mapviewer.services.QueryException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IProjectService;

public class ModelRestImplTest extends RestTestFunctions {

  @Autowired
  ModelRestImpl _modelRestImpl;

  @Autowired
  IModelService modelService;

  @Autowired
  IProjectService projectService;

  @Autowired
  ProjectDao projectDao;

  Model map;

  @Before
  public void setUp() throws Exception {
    _modelRestImpl.setModelService(modelService);
    _modelRestImpl.setProjectService(projectService);
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testModels() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    List<Map<String, Object>> result = _modelRestImpl.getModels("sample");
    assertEquals(1, result.size());
  }

  @Test
  public void testGetEmptyModels() throws Exception {
    Project project = new Project("test");
    project.setOwner(userService.getUserByLogin(ADMIN_BUILT_IN_LOGIN));
    projectDao.add(project);
    List<Map<String, Object>> result = _modelRestImpl.getModels("test");
    assertEquals(0, result.size());
  }


  private ModelRestImpl createMockProjectRest(String string) throws Exception {
    map = super.getModelForFile(string, false);

    Project project = new Project(map.getName());
    project.setOwner(userService.getUserByLogin(ADMIN_BUILT_IN_LOGIN));
    project.addModel(map);
    projectDao.add(project);

    return _modelRestImpl;
  }

  @Test(expected = QueryException.class)
  public void testGetModelAsImageForInvalidConverter() throws Exception {
    ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
    modelRest.getModelAsImage("sample", "0", "", "", "", "", "", null);
  }

  @Test
  public void testGetModelAsImage() throws Exception {
    ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
    FileEntry result = modelRest.getModelAsImage("sample", "" + map.getId(), PdfImageGenerator.class.getCanonicalName(),
        "",
        "", "", "", null);
    assertNotNull(result);
  }

  @Test
  public void testGetModelAsImage2() throws Exception {
    ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
    modelRest.getModelAsImage("sample", "" + map.getId(), PdfImageGenerator.class.getCanonicalName(), "", "", "", "",
        null);
  }

  @Test(expected = QueryException.class)
  public void testGetModelAsFileModel() throws Exception {
    ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
    modelRest.getModelAsModelFile("sample", "" + map.getId(), "", "", "", null, null, null, "");
  }

  @Test
  public void testGetModelAsFileModel2() throws Exception {
    ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
    modelRest.getModelAsModelFile("sample", "" + map.getId(), CellDesignerXmlParser.class.getCanonicalName(), "",
        "0,0;90,0;90,90;90,0", null, null, null, "");
  }

  @Test(expected = QueryException.class)
  public void testGetModelAsFileModel3() throws Exception {
    ModelRestImpl modelRest = createMockProjectRest("testFiles/model/sample.xml");
    modelRest.getModelAsModelFile("sample", "0", "", "", "0,0;90,0;90,90;90,0", null, null, null, "");
  }

  @Test
  public void testEmptyStringListToIntegerList() throws Exception {
    Set<Integer> ids = _modelRestImpl.stringListToIntegerSet("");
    assertNotNull(ids);
    assertEquals(0, ids.size());
  }

  @Test
  public void testStringListToIntegerArray() throws Exception {
    Set<Integer> ids = _modelRestImpl.stringListToIntegerSet("12,3");
    assertNotNull(ids);
    assertEquals(2, ids.size());
  }
}
