package lcsb.mapviewer.api.projects.models.publications;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.*;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.api.RestTestFunctions;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.persist.dao.ProjectDao;
import lcsb.mapviewer.services.QueryException;

public class PublicationsRestImplTest extends RestTestFunctions {
  Logger logger = LogManager.getLogger();

  @Autowired
  PublicationsRestImpl projectRest;

  @Autowired
  ProjectDao projectDao;

  @AfterClass
  public static void tearDownAfterClass() throws Exception {
  }

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testPublications() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    Map<String, Object> result = projectRest.getPublications("sample", "*", "0", 20, "", "", "");
    assertEquals(1, result.get("totalSize"));
    assertEquals(0, result.get("page"));
    assertEquals(1, ((List<?>) result.get("data")).size());

    result = projectRest.getPublications("sample", "*", "1", 1, "", "", "");
    assertEquals(1, result.get("totalSize"));
    assertEquals(1, result.get("page"));
    assertEquals(0, ((List<?>) result.get("data")).size());
  }

  @Test(expected=QueryException.class)
  public void testPublicationsInvalidPageSize() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    Map<String, Object> result = projectRest.getPublications("sample", "*", "0", 0, "", "", "");
    assertEquals(1, result.get("totalSize"));
    assertEquals(0, result.get("page"));
    assertEquals(0, ((List<?>) result.get("data")).size());
  }

  @Test
  public void testPublicationsFiltering() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    Map<String, Object> result = projectRest.getPublications("sample", "*", "0", 20, "pubmedId", "asc", "123");
    assertEquals(1, result.get("totalSize"));
    assertEquals(0, result.get("page"));
    assertEquals(1, ((List<?>) result.get("data")).size());
  }

  @Test
  public void testPublicationsFilteringNonExistingPubmed() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    Map<String, Object> result = projectRest.getPublications("sample", "*", "0", 20, "pubmedId", "asc", "1234");
    assertEquals(1, result.get("totalSize"));
    assertEquals(0, result.get("page"));
    assertEquals(0, ((List<?>) result.get("data")).size());
  }

  @Test
  public void testPublicationsFilteringByModelName() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    Map<String, Object> result = projectRest.getPublications("sample", "*", "0", 20, "pubmedId", "asc",
        "sample");
    assertEquals(1, result.get("totalSize"));
    assertEquals(0, result.get("page"));
    assertTrue(((List<?>) result.get("data")).size() > 0);

    result = projectRest.getPublications("sample", "*", "0", 20, "pubmedId", "asc", "sampleX");
    assertEquals(1, result.get("totalSize"));
    assertEquals(0, result.get("page"));
    assertEquals(0, ((List<?>) result.get("data")).size());
  }

  @Test(expected = QueryException.class)
  public void testPublicationsInvalidSortingFiltering() throws Exception {
    createMockProjectRest("testFiles/model/sample.xml");
    projectRest.getPublications("sample", "*", "0", 20, "xxx", "asc", "123");
  }

  private void createMockProjectRest(String string) throws Exception {
    Model model = super.getModelForFile(string, false);
    Project project = new Project(model.getName());
    project.addModel(model);
    project.setOwner(userService.getUserByLogin(ADMIN_BUILT_IN_LOGIN));
    projectDao.add(project);
  }

}
