package lcsb.mapviewer.services.impl;

import java.util.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.internal.LinkedTreeMap;

import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.persist.dao.map.species.ElementDao;
import lcsb.mapviewer.persist.dao.map.species.ElementProperty;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.QueryException;
import lcsb.mapviewer.services.interfaces.IElementService;
import lcsb.mapviewer.services.interfaces.IModelService;

@Transactional
@Service
public class ElementService implements IElementService {

  @Autowired
  private IModelService modelService;

  @Autowired
  private ElementDao elementDao;

  @Override
  public Element getElementById(String projectId, int mapId, int elementId) throws QueryException {
    ModelData map = modelService.getModelByMapId(projectId, mapId);
    Map<ElementProperty, List<?>> properties = new HashMap<>();
    properties.put(ElementProperty.ID, Arrays.asList(elementId));
    properties.put(ElementProperty.MAP, Arrays.asList(map));
    Page<Element> elements = elementDao.getByFilter(Pageable.unpaged(), properties);
    if (elements.getNumberOfElements() > 0) {
      return elements.getContent().get(0);
    }
    throw new ObjectNotFoundException("Element does not exist");
  }

  @Override
  public Element getElementById(String projectId, int elementId) throws ObjectNotFoundException {
    List<ModelData> maps = modelService.getModelsByProjectId(projectId);
    if (maps.size() == 0) {
      throw new ObjectNotFoundException("Element does not exist");
    }
    Map<ElementProperty, List<?>> properties = new HashMap<>();
    properties.put(ElementProperty.ID, Arrays.asList(elementId));
    properties.put(ElementProperty.MAP, maps);
    Page<Element> elements = elementDao.getByFilter(Pageable.unpaged(), properties);
    if (elements.getNumberOfElements() > 0) {
      return elements.getContent().get(0);
    }
    throw new ObjectNotFoundException("Element does not exist");
  }

  @Override
  public Element getById(int id) {
    return elementDao.getById(id);
  }

  public List<Element> getElementsByFilter(Map<ElementProperty, List<? extends Object>> properties) {
    return elementDao.getByFilter(Pageable.unpaged(), properties).getContent();
  }

  @Override
  public Map<MiriamType, Integer> getAnnotationStatistics(String projectId, String mapId) throws QueryException {
    Map<MiriamType, Integer> elementAnnotations = new LinkedTreeMap<>();
    Map<MiriamType, Integer> data = elementDao.getAnnotationStatistics(modelService.getModelsByMapId(projectId, mapId));
    for (MiriamType mt : MiriamType.values()) {
      if (data.get(mt) != null) {
        elementAnnotations.put(mt, data.get(mt));
      } else {
        elementAnnotations.put(mt, 0);
      }
    }
    return elementAnnotations;
  }

}
