package lcsb.mapviewer.services.impl;

import java.util.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.gson.internal.LinkedTreeMap;

import lcsb.mapviewer.model.map.MiriamType;
import lcsb.mapviewer.model.map.kinetics.SbmlParameter;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.persist.dao.map.ReactionDao;
import lcsb.mapviewer.persist.dao.map.ReactionProperty;
import lcsb.mapviewer.services.QueryException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IReactionService;

@Transactional
@Service
public class ReactionService implements IReactionService {

  @Autowired
  private IModelService modelService;

  @Autowired
  private ReactionDao reactionDao;

  @Override
  public Reaction getReactionById(String projectId, int mapId, int elementId) throws QueryException {
    ModelData map = modelService.getModelByMapId(projectId, mapId);
    return reactionDao.getById(elementId, map);
  }

  @Override
  public Reaction getById(int id) {
    return reactionDao.getById(id);
  }

  @Override
  public List<Reaction> getReactionById(String projectId, String mapId, Set<Integer> ids,
      Set<Integer> participantIds) throws QueryException {
    List<ModelData> maps = modelService.getModelsByMapId(projectId, mapId);
    return reactionDao.getByIds(ids, participantIds, maps);
  }

  @Override
  public List<SbmlParameter> getParameters(String projectId, String mapId) throws QueryException {
    List<ModelData> maps = modelService.getModelsByMapId(projectId, mapId);
    return reactionDao.getParametersByMaps(maps);
  }

  @Override
  public Map<MiriamType, Integer> getAnnotationStatistics(String projectId, String mapId) throws QueryException {
    Map<MiriamType, Integer> elementAnnotations = new LinkedTreeMap<>();
    Map<MiriamType, Integer> data = reactionDao
        .getAnnotationStatistics(modelService.getModelsByMapId(projectId, mapId));
    for (MiriamType mt : MiriamType.values()) {
      if (data.get(mt) != null) {
        elementAnnotations.put(mt, data.get(mt));
      } else {
        elementAnnotations.put(mt, 0);
      }
    }
    return elementAnnotations;
  }

  @Override
  public Page<Reaction> getReactionsByFilter(Map<ReactionProperty, List<? extends Object>> reactionSearch) {
    return reactionDao.getByFilter(Pageable.unpaged(), reactionSearch);
  }

}
