package lcsb.mapviewer.services.interfaces;

import java.util.List;

import lcsb.mapviewer.common.IProgressUpdater;
import lcsb.mapviewer.model.map.kinetics.SbmlParameter;
import lcsb.mapviewer.model.map.kinetics.SbmlUnit;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.QueryException;

/**
 * Service that manages models (maps).
 * 
 * @author Piotr Gawron
 * 
 */
public interface IModelService {

  /**
   * Caches information about all pubmed articles for the given model.
   * 
   * @param model
   *          model in which we are caching the data
   * @param updater
   *          callback interface that update progress information
   */
  void cacheAllPubmedIds(Model model, IProgressUpdater updater);

  /**
   * Caches information about all miriam resources in the model.
   * 
   * @param model
   *          model in which we are caching the data
   * @param updater
   *          callback interface that update progress information
   */
  void cacheAllMiriamLinks(Model model, IProgressUpdater updater);

  void updateModel(ModelData model);

  List<ModelData> getModelsByProjectId(String projectId) throws ObjectNotFoundException;

  List<ModelData> getModelsByMapId(String projectId, String mapId) throws QueryException;

  ModelData getModelByMapId(String projectId, int mapId) throws QueryException;

  Model getAndFetchModelByMapId(String projectId, int mapId) throws QueryException;

  List<SbmlParameter> getParametersForModel(String projectId, String mapId) throws QueryException;

  int getPublicationCount(String projectId, String mapId) throws QueryException;

  List<SbmlUnit> getUnitsForModel(String projectId, String modelId) throws QueryException;
}
