package lcsb.mapviewer.services.interfaces;

import java.awt.geom.Point2D;
import java.util.Collection;
import java.util.List;

import lcsb.mapviewer.model.map.BioEntity;
import lcsb.mapviewer.services.QueryException;

/**
 * Service that gives access to search functionalities.
 * 
 * @author Piotr Gawron
 * 
 */
public interface ISearchService {

  /**
   * Search for elements on the map by query. Returns many possibilities from best
   * matching to less matching possibility.
   * 
   * @param model
   *          model on which we perform search
   * @param query
   *          the query
   * @param limit
   *          maximum number of results
   * @param perfectMatch
   *          should the match be perfect
   * @param ipAddress
   *          ip address of a client who is searching (used for logging purpose)
   * @return list of objects that matches the query sorted by the match quality
   * @throws QueryException
   */
  List<BioEntity> searchByQuery(String projectId, String mapId, String query, int limit, boolean perfectMatch,
      String ipAddress) throws QueryException;

  List<BioEntity> searchByQuery(String projectId, String mapId, String query, int limit, boolean perfectMatch)
      throws QueryException;

  /**
   * Returns the closest elements to the coordinates on the model.
   * 
   * @param model
   *          model on which the search is performed
   * @param point
   *          coordinates where search is performed
   * @param numberOfElements
   *          how many closest elements should be returned
   * @return list of the closest elements
   * @throws QueryException
   */
  List<BioEntity> getClosestElements(String projectId, int mapId, Point2D point, int numberOfElements,
      boolean perfectHit,
      Collection<String> types) throws QueryException;

  List<String> getSuggestedQueryList(String projectId) throws QueryException;

}
