package lcsb.mapviewer.web;

import static org.springframework.restdocs.mockmvc.MockMvcRestDocumentation.document;
import static org.springframework.restdocs.mockmvc.RestDocumentationRequestBuilders.get;
import static org.springframework.restdocs.payload.PayloadDocumentation.*;
import static org.springframework.restdocs.request.RequestDocumentation.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.*;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockHttpSession;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.web.servlet.RequestBuilder;

import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.interfaces.IProjectService;

@RunWith(SpringJUnit4ClassRunner.class)
public class PublicationsControllerIntegrationTest extends ControllerIntegrationTest {

  Logger logger = LogManager.getLogger();

  @Autowired
  private IProjectService projectService;

  @Autowired
  private IModelService modelService;

  private Project project;
  private ModelData map;

  @Before
  public void setup() throws ObjectNotFoundException {
    project = projectService.getProjectByProjectId(BUILT_IN_PROJECT);
    map = modelService.getModelsByProjectId(BUILT_IN_PROJECT).get(0);
  }

  @After
  public void tearDown() throws Exception {
    removeProjectInSeparateThread(TEST_PROJECT);
  }

  @Test
  public void testGetPublicationsForMap() throws Exception {
    MockHttpSession session = createSession(BUILT_IN_TEST_ADMIN_LOGIN, BUILT_IN_TEST_ADMIN_PASSWORD);
    project = createProjectInSeparateThread(TEST_PROJECT);
    map = project.getModels().iterator().next();

    RequestBuilder request = get("/projects/{projectId}/models/{mapId}/publications/", TEST_PROJECT, map.getId())
        .session(session);

    mockMvc.perform(request)
        .andDo(document("projects/project_data/get_publications",
            requestParameters(
                parameterWithName("start").description("number of first entry in this response").optional(),
                parameterWithName("sortColumn")
                    .description("entry column that should be used for sorting (default: pubmedId)").optional(),
                parameterWithName("sortOrder").description("entry sort order (asc, desc)").optional(),
                parameterWithName("length").description("number of entres we want to obtain").optional(),
                parameterWithName("search").description("search query used for filtering").optional()),
            pathParameters(parameterWithName("projectId").description("project identifier"),
                parameterWithName("mapId").description("submap identifier (use '*' to indicate all maps)")),
            responseFields(
                subsectionWithPath("data")
                    .description("list of publications")
                    .type("array<Article>"),
                fieldWithPath("filteredSize")
                    .description("number of entries that match filter criteria")
                    .type("number"),
                fieldWithPath("length")
                    .description("number of entries in this response")
                    .type("number"),
                fieldWithPath("page")
                    .description("page number of result list")
                    .type("number"),
                fieldWithPath("totalSize")
                    .description("number of all log entries")
                    .type("number"))))
        .andExpect(status().is2xxSuccessful());
  }

  @Test
  public void testGetPublicationsWithUndefinedProject() throws Exception {
    MockHttpSession session = createSession(BUILT_IN_TEST_ADMIN_LOGIN, BUILT_IN_TEST_ADMIN_PASSWORD);

    RequestBuilder request = get("/projects/*/models/" + map.getId() + "/publications/")
        .session(session);

    mockMvc.perform(request)
        .andExpect(status().isNotFound());
  }

}
