package lcsb.mapviewer.web;

import static org.mockito.ArgumentMatchers.anyString;

import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.*;

import lcsb.mapviewer.annotation.cache.BigFileCache;
import lcsb.mapviewer.model.cache.BigFileEntry;
import lcsb.mapviewer.persist.DbUtils;
import lcsb.mapviewer.persist.dao.ConfigurationDao;
import lcsb.mapviewer.persist.dao.cache.BigFileEntryDao;

@Profile("webBigFileAlwaysExistsTestProfile")
@Configuration
public class WebBigFileCacheConfiguration {

  @Autowired
  private BigFileEntryDao bigFileEntryDao;
  @Autowired
  private ConfigurationDao configurationDao;
  @Autowired
  private DbUtils dbUtils;

  @Bean
  @Primary
  public BigFileCache bigFileCache() throws Exception {
    BigFileCache cache = new BigFileCache(bigFileEntryDao, configurationDao, dbUtils);
    BigFileCache mock = Mockito.spy(cache);

    Mockito.doAnswer(new Answer<String>() {

      @Override
      public String answer(InvocationOnMock invocation) throws Throwable {
        Object[] args = invocation.getArguments();
        String remoteUrl = (String) args[0];
        BigFileEntry entry = bigFileEntryDao.getByUrl(remoteUrl);
        if (entry != null) {
          return entry.getLocalPath();
        } else {
          return null;
        }
      }
    }).when(mock).getLocalPathForFile(anyString());

    return mock;
  }

}
